@@ -1,23 +1,28 @@
 package Net::Google::DataAPI;
 use 5.008001;
-use Any::Moose;
-use Any::Moose '::Exporter';
+use Moose ();
+use Moose::Exporter;
 use Carp;
 use Lingua::EN::Inflect::Number qw(to_PL);
 use XML::Atom;
-our $VERSION = '0.2802_2';
+our $VERSION = '0.01';
 
-any_moose('::Exporter')->setup_import_methods(
-    as_is => ['feedurl', 'entry_has'],
+Moose::Exporter->setup_import_methods(
+    with_caller => ['feedurl', 'entry_has'],
 );
 
 sub feedurl {
-    my ($name, %args) = @_;
+    my ($caller, $name, %args) = @_;
 
-    my $class = caller;
+    my $class = Class::MOP::class_of($caller);
+    $class->does_role('Net::Google::DataAPI::Role::Entry') 
+        or $class->does_role('Net::Google::DataAPI::Role::Service')
+        or confess 'Net::Google::DataAPI::Role::(Service|Entry) required to use feedurl';
 
     my $entry_class = delete $args{entry_class} 
         or confess 'entry_class not specified';
+    Class::MOP::load_class($entry_class)->does_role('Net::Google::DataAPI::Role::Entry')
+        or confess "$entry_class should do Net::Google::DataAPI::Role::Entry role";
 
     my $can_add = delete $args{can_add};
     $can_add = 1 unless defined $can_add;
@@ -37,49 +42,27 @@ sub feedurl {
     my $from_atom = delete $args{from_atom};
     my $rel = delete $args{rel};
     my $as_content_src = delete $args{as_content_src};
-    my $default = delete $args{default} || '';
 
     my $attr_name = "${name}_feedurl";
 
-    my $class_meta = any_moose('::Meta::Class')->initialize($class);
-    $class_meta->add_attribute(
+    $class->add_attribute(
         $attr_name => (
             isa => 'Str',
             is => 'ro',
-            lazy_build => 1,
             %args,
         )
     );
-    $class_meta->add_method(
-        "${name}_entryclass" => sub { $entry_class }
-    );
-
-    $class_meta->add_method(
-        "_build_$attr_name" => sub {
-            my $self = shift;
-            return $rel ?  
-            [
-            map { $_->href }
-            grep { $_->rel eq $rel }
-            $self->atom->link
-            ]->[0] :
-            $as_content_src ? 
-            $self->atom->content->elem->getAttribute('src') :
-            $from_atom ?
-            $from_atom->($self, $self->atom) : $default;
-        }
-    );
     my $pl_name = to_PL($name);
 
     if ($can_add) {
-        $class_meta->add_method(
+        $class->add_method(
             "add_$name" => sub {
                 my ($self, $args) = @_;
                 $self->$attr_name or confess "$attr_name is not set";
-                Any::Moose::load_class($entry_class);
+                Class::MOP::load_class($entry_class);
                 $args = $arg_builder->($self, $args);
                 my %parent = 
-                    $self->can('sync') ?
+                    $class->does_role('Net::Google::DataAPI::Role::Entry') ?
                     ( container => $self ) : ( service => $self );
                 my $entry = $entry_class->new(
                     {
@@ -88,7 +71,7 @@ sub feedurl {
                     }
                 )->to_atom;
                 my $atom = $self->service->post($self->$attr_name, $entry);
-                $self->sync if $self->can('sync');
+                $self->sync if $class->does_role('Net::Google::DataAPI::Role::Entry');
                 my $e = $entry_class->new(
                     %parent,
                     atom => $atom,
@@ -97,50 +80,54 @@ sub feedurl {
             }
         );
     }
-    $class_meta->add_method(
+    $class->add_method(
         $pl_name => sub {
             my ($self, $cond) = @_;
-            my $feed = do {
-                if (ref($cond) eq 'XML::Atom::Feed') {
-                    $cond;
-                } else {
-                    $self->$attr_name or confess "$attr_name is not set";
-                    Any::Moose::load_class($entry_class);
-                    $self->can("${name}_feed")->($self, $cond);
-                }
-            };
+            $self->$attr_name or confess "$attr_name is not set";
+            Class::MOP::load_class($entry_class);
+            $cond = $query_builder->($self, $cond);
+            my $feed = $self->service->get_feed($self->$attr_name, $cond);
             return map {
                 $entry_class->new(
-                    $self->can('sync') ?
+                    $class->does_role('Net::Google::DataAPI::Role::Entry') ?
                     ( container => $self ) : ( service => $self ),
                     atom => $_,
                 )
             } $feed->entries;
         }
     );
-    $class_meta->add_method(
-        "${name}_feed" => sub {
-            my ($self, $cond) = @_;
-            $self->$attr_name or confess "$attr_name is not set";
-            $cond = $query_builder->($self, $cond);
-            return $self->service->get_feed($self->$attr_name, $cond);
-        }
-    );
-    $class_meta->add_method(
+    $class->add_method(
         $name => sub {
             my ($self, $cond) = @_;
             return [ $self->$pl_name($cond) ]->[0];
         }
     );
 
+    if ( $class->find_method_by_name('from_atom') ) {
+        $class->add_after_method_modifier(
+            'from_atom' => sub {
+                my ($self) = @_;
+                $self->{$attr_name} = 
+                    $rel ?  
+                        [
+                            map { $_->href }
+                            grep { $_->rel eq $rel }
+                            $self->atom->link
+                        ]->[0] :
+                    $as_content_src ? 
+                        $self->atom->content->elem->getAttribute('src') :
+                    $from_atom ?
+                        $from_atom->($self->atom) : undef;
+            }
+        );
+    }
 }
 
 sub entry_has {
-    my ($name, %args) = @_;
+    my ($caller, $name, %args) = @_;
 
-    my $class = caller;
-    my $class_meta = any_moose('::Meta::Class')->initialize($class);
-    $class_meta->does_role('Net::Google::DataAPI::Role::Entry') 
+    my $class = Class::MOP::class_of($caller);
+    $class->does_role('Net::Google::DataAPI::Role::Entry') 
         or confess 'Net::Google::DataAPI::Role::Entry required to use entry_has';
 
     my $tagname = delete $args{tagname};
@@ -148,24 +135,18 @@ sub entry_has {
 
     my $from_atom = delete $args{from_atom};
     my $to_atom = delete $args{to_atom};
-    my $default = delete $args{default} || '';
-    $default = $default->() if ref $default eq 'CODE';
 
-    $class_meta->add_attribute(
+    $class->add_attribute(
         $name => (
             isa => 'Str',
             is => 'ro',
-            $to_atom || $tagname ?  (
-                trigger => sub {$_[0]->update }
-            ) : (),
-            $tagname || $from_atom ? (
-                lazy_build => 1,
-            ) : (),
+            $to_atom || $tagname ? 
+                (trigger => sub {$_[0]->update }) : (),
             %args,
         )
     );
     if ($tagname) {
-        $class_meta->add_around_method_modifier(
+        $class->add_around_method_modifier(
             to_atom => sub {
                 my ($next, $self) = @_;
                 my $entry = $next->($self);
@@ -174,17 +155,16 @@ sub entry_has {
                 return $entry;
             }
         );
-        $class_meta->add_method(
-            "_build_$name" => sub {
+        $class->add_after_method_modifier(
+            from_atom => sub {
                 my $self = shift;
-                $self->atom or return $default;
                 my $ns_obj = $ns ? $self->ns($ns) : $self->atom->ns;
-                return $self->atom->get($ns_obj, $tagname) || $default;
+                $self->{$name} = $self->atom->get($ns_obj, $tagname);
             }
         );
     }
     if ($to_atom) {
-        $class_meta->add_around_method_modifier(
+        $class->add_around_method_modifier(
             to_atom => sub {
                 my ($next, $self) = @_;
                 my $entry = $next->($self);
@@ -194,20 +174,15 @@ sub entry_has {
         );
     }
     if ($from_atom) {
-        $class_meta->add_method(
-            "_build_$name" => sub {
+        $class->add_after_method_modifier(
+            from_atom => sub {
                 my $self = shift;
-                $self->atom or return $default;
-                return $from_atom->($self, $self->atom) || $default;
+                $self->{$name} = $from_atom->($self, $self->atom);
             }
         );
     }
 }
 
-__PACKAGE__->meta->make_immutable;
-no Any::Moose;
-no Any::Moose '::Exporter';
-
 1;
 __END__
 
@@ -218,17 +193,19 @@ Net::Google::DataAPI - Base implementations for modules to negotiate with Google
 =head1 SYNOPSIS
 
   package MyService;
-  use Any::Moose;
+  use Moose;
   use Net::Google::DataAPI;
 
-  with 'Net::Google::DataAPI::Role::Service';
-
-  # registering xmlns
-  has '+namespaces' => (
-    default => {
-        foobar => 'http://example.com/schema#foobar',
-    },
-  );
+  with 'Net::Google::DataAPI::Role::Service' => {
+      service => 'foobar', 
+        # see http://code.google.com/intl/ja/apis/gdata/faq.html#clientlogin
+      source => __PACKAGE__,
+        # source name to pass to Net::Google::AuthSub
+      ns => {
+          foobar => 'http://example.com/schema#foobar',
+      }
+        # registering xmlns
+  };
 
   # registering feed url
   feedurl myentry => (
@@ -237,22 +214,10 @@ Net::Google::DataAPI - Base implementations for modules to negotiate with Google
       default => 'http://example.com/myfeed',
   );
 
-  sub _build_auth {
-      my ($self) = @_;
-      # .. authsub login things, this is optional.
-      # see Net::Google::Spreadsheets for typical implementation
-      my $authsub = Net::Google::DataAPI::Auth::AuthSub->new(
-          service => 'wise',
-          account_type => 'HOSTED_OR_GOOGLE',
-      );
-      $authsub->login('foo.bar@gmail.com', 'p4ssw0rd');
-      return $authsub;
-  }
-
   1;
 
   package MyEntry;
-  use Any::Moose;
+  use Moose;
   use Net::Google::DataAPI;
   with 'Net::Google::DataAPI::Role::Entry';
 
@@ -281,13 +246,9 @@ define a feed url.
 
 define a entry attribute.
 
-=head1 DEBUGGING
-
-You can set environment variable GOOGLE_DATAAPI_DEBUG=1 to see the raw requests and responses Net::Google::DataAPI sends and receives.
-
 =head1 AUTHOR
 
-Nobuo Danjou E<lt>danjou@soffritto.orgE<gt>
+Nobuo Danjou E<lt>nobuo.danjou@gmail.comE<gt>
 
 =head1 TODO
 
@@ -297,12 +258,6 @@ more pods.
 
 L<Net::Google::AuthSub>
 
-L<Net::Google::DataAPI::Auth::AuthSub>
-
-L<Net::Google::DataAPI::Auth::ClientLogin::Multiple>
-
-L<Net::Google::DataAPI::Auth::OAuth>
-
 L<Net::Google::DataAPI::Role::Service>
 
 L<Net::Google::DataAPI::Role::Entry>
